import numpy as np

import npx


def test_isin():
    a = [[0, 3], [1, 0]]
    b = [[1, 0], [7, 12], [-1, 5]]

    out = npx.isin_rows(a, b)
    assert np.all(out == [False, True])


def test_scalar():
    a = [0, 3, 5]
    b = [-1, 6, 5, 0, 0, 0]

    out = npx.isin_rows(a, b)
    assert np.all(out == [True, False, True])



import numpy as np

import npx

np.random.seed(0)


def test_dot():
    a = np.random.rand(1, 2, 3)
    b = np.random.rand(3, 4, 5)
    c = npx.dot(a, b)
    assert c.shape == (1, 2, 4, 5)


def test_solve():
    a = np.random.rand(3, 3)
    b = np.random.rand(3, 4, 5)
    c = npx.solve(a, b)
    assert c.shape == b.shape



import numpy as np

import npx


def test_sum_at():
    a = [1.0, 2.0, 3.0]
    idx = [0, 1, 0]
    out = npx.sum_at(a, idx, minlength=4)

    tol = 1.0e-13
    ref = np.array([4.0, 2.0, 0.0, 0.0])
    assert np.all(np.abs(out - ref) < (1 + np.abs(ref)) * tol)


def test_add_at():
    a = [1.0, 2.0, 3.0]
    idx = [0, 1, 0]
    out = np.zeros(2)
    npx.add_at(out, idx, a)

    tol = 1.0e-13
    ref = np.array([4.0, 2.0])
    assert np.all(np.abs(out - ref) < (1 + np.abs(ref)) * tol)


def test_subtract_at():
    a = [1.0, 2.0, 3.0]
    idx = [0, 1, 0]
    out = np.ones(2)
    npx.subtract_at(out, idx, a)

    tol = 1.0e-13
    ref = np.array([-3.0, -1.0])
    assert np.all(np.abs(out - ref) < (1 + np.abs(ref)) * tol)



from .__about__ import __version__
from ._isin import isin_rows
from ._main import add_at, dot, solve, subtract_at, sum_at, unique_rows

__all__ = [
    "__version__",
    "dot",
    "solve",
    "sum_at",
    "add_at",
    "subtract_at",
    "unique_rows",
    "isin_rows",
]



import numpy as np


def isin_rows(a, b):
    a = np.asarray(a)
    b = np.asarray(b)
    if not np.issubdtype(a.dtype, np.integer):
        raise ValueError(f"Input array must be integer type, got {a.dtype}.")
    if not np.issubdtype(b.dtype, np.integer):
        raise ValueError(f"Input array must be integer type, got {b.dtype}.")

    a = a.reshape(a.shape[0], np.prod(a.shape[1:], dtype=int))
    b = b.reshape(b.shape[0], np.prod(b.shape[1:], dtype=int))

    a_view = np.ascontiguousarray(a).view(
        np.dtype((np.void, a.dtype.itemsize * a.shape[1]))
    )
    b_view = np.ascontiguousarray(b).view(
        np.dtype((np.void, b.dtype.itemsize * b.shape[1]))
    )

    out = np.isin(a_view, b_view)

    return out.reshape(a.shape[0])
